==> karlender-v0.6.0.tar.gz: karlender-v0.6.0/src/widgets/smart/month_cell.rs <==
use crate::domain::*;
use crate::store::{store, State};
use crate::widgets::EventTag;
use gdk4::subclass::prelude::ObjectSubclassIsExt;
use glib::subclass::signal::Signal;
use glib::SignalHandlerId;
use gtk::builders::GestureSwipeBuilder;
use gtk::{prelude::*, GestureClick, GestureSwipe};
use gtk_rust_app::gstore::dispatch;
use libadwaita as adw;
use std::cell::Cell;

#[widget(@store extends gtk::Box)]
#[template(file = "month_cell.ui")]
struct MonthCell {
    #[template_child]
    pub cell_leaflet: TemplateChild<adw::Leaflet>,
    #[template_child]
    pub day_label: TemplateChild<gtk::Label>,
    #[template_child]
    pub event_leaflet: TemplateChild<adw::Leaflet>,

    #[property_bool]
    pub current_month: Cell<bool>,
    #[property_string]
    pub date: Cell<String>,

    #[signal(Signal::builder("clicked", &[i32::static_type().into()], <()>::static_type().into()).build())]
    clicked: (),

    #[selector(state.calendar_slice)]
    select_calendar_slice: (),

    pub event_leaflet_signal_handlers: Cell<Vec<SignalHandlerId>>,

    pub gesture_handler: Cell<Option<(GestureClick, SignalHandlerId)>>,
    pub swipe_handler: Cell<Option<(GestureSwipe, SignalHandlerId)>>,

    #[dispose]
    dispose: (),
}

impl MonthCell {
    pub fn new(date: KarlenderDate, current_month: bool) -> Self {
        let month_cell: Self = glib::Object::new(&[
            ("current-month", &current_month),
            ("date", &date.and_hms(0, 0, 0, None).to_string()), //
        ])
        .expect("Failed to create MonthGrid");
        month_cell
    }

    pub fn constructed(&self) {
        let s = self;

        self.auto_cleanup_signal_handler(self.connect_realize(|s| s.realized()));

        let gesture_click = GestureClick::builder().build();
        let id = gesture_click.connect_released(glib::clone!(@weak s => move |_, n_press, _, _| {
            s.emit_by_name("clicked", &[&n_press.to_value()])
        }));

        let events_leaflet: &adw::Leaflet = self.imp().event_leaflet.as_ref();
        self.imp().cell_leaflet.set_visible_child(events_leaflet);
        self.add_controller(&gesture_click);
        self.imp().gesture_handler.set(Some((gesture_click, id)));

        let swipe = GestureSwipeBuilder::new().build();
        let _self = self;
        let id = swipe.connect_swipe(glib::clone!(@weak _self => move |_gs, x, _y| {
            if x > 400. {
                dispatch!(crate::store::_GO_PREV)
            } else if x < -400. {
                dispatch!(crate::store::_GO_NEXT)
            }
        }));

        self.add_controller(&swipe);
        self.imp().swipe_handler.set(Some((swipe, id)));
    }

    pub fn dispose(&self) {
        let signal_handlers = self.imp().event_leaflet_signal_handlers.take();
        for signal in signal_handlers {
            self.event_leaflet().disconnect(signal);
        }
        if let Some((controller, id)) = self.imp().gesture_handler.take() {
            controller.disconnect(id);
            self.remove_controller(&controller);
        }
        if let Some((controller, id)) = self.imp().swipe_handler.take() {
            controller.disconnect(id);
            self.remove_controller(&controller);
        }
    }

    pub fn date(&self) -> Option<KarlenderDate> {
        self.try_property::<String>("date")
            .ok()
            .and_then(|d| KarlenderDateTime::from_str(&d).map(|d| d.date()).ok())
    }

    pub fn realized(&self) {
        if let Some(date) = self.date() {
            self.day_label().set_label(&format!("{}", date.day()));
            self.set_styles();
        } else {
            error!("Failed to load month cell: 'date' property could not be parsed!");
        }
    }

    pub fn set_styles(&self) {
        if let Some(date) = self.date() {
            let current_month: bool = self.property("current-month");
            if !current_month {
                self.style_context().add_class("cell-other");
                let last = get_month_len(&date);
                if date.day() == 1 {
                    self.style_context().add_class("border-l-1");
                } else if date.day() == last {
                    self.style_context().add_class("border-r-1");
                }
                if date.day() > 20 {
                    self.style_context().add_class("border-b-1");
                } else {
                    self.style_context().add_class("border-t-1");
                }
            } else {
                self.style_context().add_class("view");
            }

            if date == KarlenderTimezone::local().today() {
                self.style_context().add_class("cell-current");
            }
        }
    }

    pub fn connect_clicked(&self, f: impl Fn(&Self, i32) + 'static) {
        let id = self.connect_closure(
            "clicked",
            false,
            closure_local!(move |s: Self, num_clicks: i32| {
                f(&s, num_clicks);
            }),
        );
        self.auto_cleanup_signal_handler(id);
    }

    fn select_calendar_slice(&self, state: &State) {
        if let Some(date) = self.date() {
            let event_leaflet = self.event_leaflet();
            let calendars = &state.calendar_slice.calendars;

            let mut event_leaflet_signal_handlers = self.imp().event_leaflet_signal_handlers.take();
            for signal in event_leaflet_signal_handlers {
                event_leaflet.disconnect(signal);
            }
            event_leaflet_signal_handlers = Vec::new();
            let mut child = event_leaflet.first_child();
            while child.is_some() {
                let c = child.unwrap();
                let next = c.next_sibling();
                event_leaflet.remove(&c);
                child = next;
            }

            let mut events = Vec::new();
            for calendar in calendars.values() {
                if !calendar.enabled {
                    continue;
                }
                for event in calendar.events.values() {
                    if event.contains_date(&date) {
                        events.push((event, calendar.color))
                    }
                }
            }
            events.sort_by(|(a, _), (b, _)| Event::sort(a, b));

            let events_len = events.len() as u64;
            for (event, color) in events {
                let tag = EventTag::new(event.name.clone(), events_len, color, false);
                let id = event_leaflet.connect_folded_notify(glib::clone!(@weak tag => move |l| {
                    tag.set_folded(l.is_folded());
                }));
                event_leaflet_signal_handlers.push(id);
                event_leaflet.append(&tag);
            }

            self.imp()
                .event_leaflet_signal_handlers
                .set(event_leaflet_signal_handlers);
        }
    }
}

==> karlender-v0.9.1.tar.gz: karlender-v0.9.1/src/widgets/smart/month_cell.rs <==
use crate::domain::*;
use crate::store::{store, State};
use crate::widgets::EventTag;
use gdk4::subclass::prelude::ObjectSubclassIsExt;
use glib::subclass::signal::Signal;
use glib::SignalHandlerId;
use gtk::builders::GestureSwipeBuilder;
use gtk::{prelude::*, GestureClick, GestureSwipe};
use gtk_rust_app::gstore::dispatch;
use libadwaita as adw;
use std::cell::Cell;

#[widget(@store extends gtk::Box)]
#[template(file = "month_cell.ui")]
struct MonthCell {
    #[template_child]
    pub cell_leaflet: TemplateChild<adw::Leaflet>,
    #[template_child]
    pub day_label: TemplateChild<gtk::Label>,
    #[template_child]
    pub event_leaflet: TemplateChild<adw::Leaflet>,

    #[property_bool]
    pub current_month: Cell<bool>,
    #[property_string]
    pub date: Cell<String>,

    #[signal(Signal::builder("clicked", &[i32::static_type().into()], <()>::static_type().into()).build())]
    clicked: (),

    #[selector(state.calendar_slice)]
    select_calendar_slice: (),

    pub event_leaflet_signal_handlers: Cell<Vec<SignalHandlerId>>,

    pub gesture_handler: Cell<Option<(GestureClick, SignalHandlerId)>>,
    pub swipe_handler: Cell<Option<(GestureSwipe, SignalHandlerId)>>,

    #[dispose]
    dispose: (),
}

impl MonthCell {
    pub fn new(date: KarlenderDate, current_month: bool) -> Self {
        let month_cell: Self = glib::Object::new(&[
            ("current-month", &current_month),
            ("date", &date.and_hms(0, 0, 0, None).to_string()), //
        ])
        .expect("Failed to create MonthGrid");
        month_cell
    }

    pub fn constructed(&self) {
        let s = self;

        self.auto_cleanup_signal_handler(self.connect_realize(|s| s.realized()));

        let gesture_click = GestureClick::builder().build();
        let id = gesture_click.connect_released(glib::clone!(@weak s => move |_, n_press, _, _| {
            s.emit_by_name("clicked", &[&n_press.to_value()])
        }));

        let events_leaflet: &adw::Leaflet = self.imp().event_leaflet.as_ref();
        self.imp().cell_leaflet.set_visible_child(events_leaflet);
        self.add_controller(&gesture_click);
        self.imp().gesture_handler.set(Some((gesture_click, id)));

        let swipe = GestureSwipeBuilder::new().build();
        let _self = self;
        let id = swipe.connect_swipe(glib::clone!(@weak _self => move |_gs, x, _y| {
            if x > 400. {
                dispatch!(crate::store::_GO_PREV)
            } else if x < -400. {
                dispatch!(crate::store::_GO_NEXT)
            }
        }));

        self.add_controller(&swipe);
        self.imp().swipe_handler.set(Some((swipe, id)));
    }

    pub fn dispose(&self) {
        let signal_handlers = self.imp().event_leaflet_signal_handlers.take();
        for signal in signal_handlers {
            self.event_leaflet().disconnect(signal);
        }
        if let Some((controller, id)) = self.imp().gesture_handler.take() {
            controller.disconnect(id);
            self.remove_controller(&controller);
        }
        if let Some((controller, id)) = self.imp().swipe_handler.take() {
            controller.disconnect(id);
            self.remove_controller(&controller);
        }
    }

    pub fn date(&self) -> Option<KarlenderDate> {
        self.try_property::<String>("date")
            .ok()
            .and_then(|d| KarlenderDateTime::from_str(&d).map(|d| d.date()).ok())
    }

    pub fn realized(&self) {
        if let Some(date) = self.date() {
            self.day_label().set_label(&format!("{}", date.day()));
            self.set_styles();
        } else {
            error!("Failed to load month cell: 'date' property could not be parsed!");
        }
    }

    pub fn set_styles(&self) {
        if let Some(date) = self.date() {
            let current_month: bool = self.property("current-month");
            if !current_month {
                self.style_context().add_class("cell-other");
                let last = get_month_len(&date);
                if date.day() == 1 {
                    self.style_context().add_class("border-l-1");
                } else if date.day() == last {
                    self.style_context().add_class("border-r-1");
                }
                if date.day() > 20 {
                    self.style_context().add_class("border-b-1");
                } else {
                    self.style_context().add_class("border-t-1");
                }
            } else {
                self.style_context().add_class("view");
            }

            if date == KarlenderTimezone::local().today() {
                self.style_context().add_class("cell-current");
            }
        }
    }

    pub fn connect_clicked(&self, f: impl Fn(&Self, i32) + 'static) {
        let id = self.connect_closure(
            "clicked",
            false,
            closure_local!(move |s: Self, num_clicks: i32| {
                f(&s, num_clicks);
            }),
        );
        self.auto_cleanup_signal_handler(id);
    }

    fn select_calendar_slice(&self, state: &State) {
        if let Some(date) = self.date() {
            let event_leaflet = self.event_leaflet();
            let calendars = &state.calendar_slice.calendars;

            let mut event_leaflet_signal_handlers = self.imp().event_leaflet_signal_handlers.take();
            for signal in event_leaflet_signal_handlers {
                event_leaflet.disconnect(signal);
            }
            event_leaflet_signal_handlers = Vec::new();
            let mut child = event_leaflet.first_child();
            while child.is_some() {
                let c = child.unwrap();
                let next = c.next_sibling();
                event_leaflet.remove(&c);
                child = next;
            }

            let mut events = Vec::new();
            for calendar in calendars.values() {
                if !calendar.enabled {
                    continue;
                }
                for event in calendar.events.values() {
                    if event.contains_date(&date) {
                        events.push((event, &calendar.color))
                    }
                }
            }
            events.sort_by(|(a, _), (b, _)| Event::sort(a, b));

            let events_len = events.len() as u64;
            for (event, color) in events {
                let tag = EventTag::new(event.name.clone(), events_len, color, false);
                let id = event_leaflet.connect_folded_notify(glib::clone!(@weak tag => move |l| {
                    tag.set_folded(l.is_folded());
                }));
                event_leaflet_signal_handlers.push(id);
                event_leaflet.append(&tag);
            }

            self.imp()
                .event_leaflet_signal_handlers
                .set(event_leaflet_signal_handlers);
        }
    }
}
